from django.test import TestCase


class TestFirewall(TestCase):
    def test_firewall(self):
        response = self.client.get("/admin/", HTTP_X_FORWARDED_FOR="83.4.12.1")
        self.assertEqual(response.status_code, 403)

    def test_multiple_ips(self):
        response = self.client.get("/admin/", HTTP_X_FORWARDED_FOR="83.4.12.1, 10.2.1.1")
        self.assertEqual(response.status_code, 403)

    def test_multiple_ips_no_x_forwarded_for_header(self):
        response = self.client.get("/admin/")
        self.assertEqual(response.status_code, 403)



#!/usr/bin/env python3

from io import open

from setuptools import find_packages, setup


def readall(path):
    with open(path, encoding="utf-8") as fp:
        return fp.read()


setup(
    name="django_simple_ip_restrict",
    version="1.0.46",
    description="Apply an IP restriction to specific routes",
    long_description=readall("README.md"),
    long_description_content_type="text/markdown",
    author="STITCH (Aleksey Panov, Mario Brito, Daniel Hengeveld, André Pereira, Kellie English)",
    author_email="panovitch@gmail.com",
    packages=find_packages(exclude=("tests.*", "tests")),
    python_requires=">=3.5",
    install_requires=["Django>=2.2", "netaddr"],
    include_package_data=True,
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Framework :: Django :: 2.2",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)



import logging
from functools import partial

from django.conf import settings
from django.http import HttpResponseForbidden
from django.urls import get_resolver, exceptions
from django.urls.resolvers import URLResolver, URLPattern, RoutePattern, RegexPattern
from netaddr import IPAddress, IPNetwork

DEFAULT_PROTECTED_NAMESPACES = ["admin"]


logger = logging.getLogger(__name__)


def ip_filter(get_response):
    protected_namespaces = frozenset(
        getattr(settings, "IP_PROTECTED_NAMESPACES", DEFAULT_PROTECTED_NAMESPACES)
    )
    whitelist = [
        IPNetwork(subnet) for subnet in getattr(settings, "IP_NETWORKS_WHITELIST", [])
    ]

    def middleware(request):
        if hasattr(request, "urlconf"):
            urlconf = request.urlconf
            set_urlconf(urlconf)
            resolver = get_resolver(urlconf)
        else:
            resolver = get_resolver()
        try:
            resolver_match = resolver.resolve(request.path_info)
        except exceptions.Resolver404:
            pass
        else:
            if not protected_namespaces.isdisjoint(resolver_match.namespaces):
                if "HTTP_X_FORWARDED_FOR" in request.META:
                    ip_address = IPAddress(request.META["HTTP_X_FORWARDED_FOR"].split(",")[0])
                else:
                    ip_address = IPAddress(request.META["REMOTE_ADDR"])
                if not any(ip_address in subnet for subnet in whitelist):
                    logger.warn(
                        "IP %s tried to access protected url %s",
                        ip_address,
                        request.path,
                    )
                    return HttpResponseForbidden()

        response = get_response(request)
        return response

    return middleware
